; Fraktale-Welt: V. Ossenkopf 2000
; Main program and routines for user interaction

; Create the opening image - not used in the fractal program
pro opening, window

wset, window
read_jpeg, "mountains.jpg", feld, colortable, colors=256
tvlct, colortable
xyouts, 0.22,0.85,/normal, "Fraktale Welt", charsize=5.0
tv, feld, 0.25,0.3, /normal
xyouts, 0.15,0.2,/normal, "fractional Brownian motion", charsize=3.0
xyouts, 0.35,0.1,/normal, "P(f) = f!U-!7b!3!N", charsize=3.0

return
end

; Check the numerical parameters
; Set them to default values if outside of a given range
function fractalcheck, string, index, value

defvals=[3.0,23.0,1.0,1.0]
minvals=[-10.0,0.0,0.25,0.125]
maxvals=[10.0,32767.0,8.0,2.0]
status=0

if (index lt 0) or (index gt 3) then begin
	status=3
	value=0.0
	return, status
endif

IF strnumber(string,value) eq 0 THEN begin
	value=defvals[index]
	status=2
ENDIF

if value gt maxvals[index] then begin
	value=maxvals[index]
	status=1
endif

if value lt minvals[index] then begin
	value=minvals[index]
	status=1
endif

RETURN, status
END

; Caller routine for the two-dimensional fractal plots
pro fractalplots, beta, seed=s, type=type, size=size, zero=zero
on_error, 2

if not keyword_set(type) then type=0
if not keyword_set(zero) then zero=1.0
if not keyword_set(size) then size=1.0
if zero eq 0.0 then zero=1.0
if size gt 8.0 then size=8.0

cscl =128
csize=600

fullsize=long(cscl*size)
if type eq 0 then begin
  fractalim, field, beta, seed=s, size=fullsize
  mmin=min(field)
  delta=max(field)-mmin
  field=((field-mmin)*20.0/delta+1.0)*zero
  loadct, 39
  !p.charsize=1.5
  overlay, field, nlevels=3, $
     title=["Rektaszension", "Deklination", "Intensitt", ""]
  !p.charsize=1.0
endif else if type eq 1 then begin
  fractalim, field, beta, seed=s, size=fullsize
  field=rebin(field,2*fullsize,2*fullsize) 
  mmin=min(field)
  delta=max(field)-mmin
  field[0,0]=mmin-0.5*delta/zero
  field[where(field gt mmin+0.75*delta)]=mmin+0.75*delta
  loadct, 1
  tvscl, congrid(field,csize,csize)
endif else begin
  fractalim, field, beta, seed=s, size=fullsize
  erase, color=255
  gebirge, field, level=zero, size=csize
endelse

return
end

; Caller routine for the sound output routines
pro sound, beta, seed=s, type=type, size=size, scal=scal, filename=filename

on_error, 2
loadct, 0

if not keyword_set(type) then type=0
if not keyword_set(scal) then scal=1.0
if not keyword_set(size) then size=1.0
if not keyword_set(filename) then filename="sound"
if scal gt 1.5 then scal=1.5
if size gt 8.0 then size=8.0

fullsize=long(128*size)
if type eq 0 then begin
  fractalfunc, field, beta, seed=s, size=fullsize
  pfield=rebin(field,fullsize,256)
  pfield=pfield(*)
  wavefile, filename, pfield, scal=scal
  pfield=[field,field,field]
  tfac=1.0/22.5
  plot, indgen(3*fullsize)*tfac, pfield, xtitle="Zeit [ms]", ytitle="Amplitude", xstyle=1
endif else if type eq 1 then begin
  fractalfunc, field, beta, seed=s, size=fullsize
  stret=512L
  mmin=min(field)
  mmax=max(field)
  field=exp((field-mmin)*3.2189*scal/(mmax-mmin)-2.9905)
  scal=float(indgen(stret))
  damp=sin((scal*!pi)/float(stret))
  pfield=fltarr(fullsize*stret)
  for j=0L,fullsize-1 do begin
      pfield[j*stret:(j+1)*stret-1]=sin(field(j)*scal)*damp
  endfor
  wavefile, filename, pfield
  fac=22.5/(2*!pi)
  tfac=float(stret)/22500.0
  plot, indgen(fullsize)*tfac,field*fac, psym=10, xtitle="Zeit [s]", ytitle="Frequenz [kHz]", xstyle=1, /ylog
endif else begin
  fullsize=8L*(fullsize/8L+1L)
  fractalfunc, field, beta, seed=s, size=fullsize
  field=rebin(field,fullsize/2)
  field=[field,field[0]]
  midifile, filename, field, 48, scal=scal
endelse

return
end

; Event handler
PRO Fractal_Event, event

normalfont="-adobe-times-medium-r-normal--14-100-100-100-p-74-iso8859-1"

IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_BUTTON') THEN BEGIN
WIDGET_CONTROL, event.id, GET_VALUE=button_value

CASE button_value OF
'Erklrung': XDISPLAYFILE, "fractalhelp.txt", font=normalfont, $
	height=30, width=76, title="Fraktale Welt - Einfuehrung", $
	DONE_BUTTON="Fertig"

'Programmende': WIDGET_CONTROL, event.top, /DESTROY
'Start': BEGIN

	WIDGET_CONTROL, event.top, GET_UVALUE=info
	WIDGET_CONTROL, info.status, SET_VALUE='Rechnung luft'
	iselect=WIDGET_INFO(info.select, /DROPLIST_SELECT)
	WIDGET_CONTROL, info.beta, GET_VALUE=sb0
	WIDGET_CONTROL, info.seed, GET_VALUE=ss0
	WIDGET_CONTROL, info.scal, GET_VALUE=sc0
	WIDGET_CONTROL, info.height, GET_VALUE=sh0
	; Numerisch
	status=''
	IF fractalcheck(sb0[0],0,b0) ne 0 THEN begin
	  status='Parameterfehler korrigiert'
	  WIDGET_CONTROL, info.beta, SET_VALUE=strtrim(string(b0,format='(F6.3)'),1)
	ENDIF
	IF fractalcheck(ss0[0],1,s0) ne 0 THEN begin
	  status='Parameterfehler korrigiert'
	  WIDGET_CONTROL, info.seed, SET_VALUE=strtrim(string(s0,format='(I6)'),1)
	ENDIF
	IF fractalcheck(sc0[0],2,c0) ne 0 THEN begin
	  status='Parameterfehler korrigiert'
	  WIDGET_CONTROL, info.scal, SET_VALUE=strtrim(string(c0,format='(F6.3)'),1)
	ENDIF
	IF fractalcheck(sh0[0],3,h0) ne 0 THEN begin
	  status='Parameterfehler korrigiert'
	  WIDGET_CONTROL, info.height, SET_VALUE=strtrim(string(h0,format='(F6.3)'),1)
	ENDIF
	; Now run the code
	WSET, info.draw_wid
	; Now run the code
	; different routine for attractor
	type=[0,1,2,2,1,0]
	if iselect gt 2 then begin
	  sndname="soundpipe"
	  sound , b0, type=type[iselect], seed=s0, size=c0, scal=h0, $
						filename=sndname
	  dim=max([min([2.5-0.5*b0,2.0]),1.0])
	endif else begin
	  fractalplots, b0, type=type[iselect], seed=s0, size=c0, zero=h0
	  dim=max([min([4.0-0.5*b0,3.0]),2.0])
	endelse
	if status eq '' then $
		status='Fraktale Dimension: '+string(dim,format='(f4.1)')
	WIDGET_CONTROL, info.status, SET_VALUE=status

	     END
ENDCASE
ENDIF

RETURN
END

; The Widget definition - Main program
PRO Fractal

boldfont="-adobe-helvetica-bold-o-normal--14-100-100-100-p-82-iso8859-1"
headfont="-adobe-helvetica-bold-r-normal--20-140-100-100-p-105-iso8859-1"
startfont="-adobe-helvetica-bold-r-normal--17-120-100-100-p-92-iso8859-1"
buttonfont="variable"
normalfont="-adobe-helvetica-medium-r-normal--14-100-100-100-p-76-iso8859-1"
tinyfont="-adobe-helvetica-medium-r-normal--10-100-75-75-p-56-iso8859-1"

tlb= WIDGET_BASE(COLUMN=2, TITLE="Fraktale Welt - V. Ossenkopf (2000)")
commands = WIDGET_BASE(tlb, COLUMN=1, /ALIGN_LEFT)
draw = WIDGET_DRAW(tlb, XSIZE=590, YSIZE=580, /ALIGN_RIGHT, COLORS=256, RETAIN=2, /BUTTON_EVENTS)

iter_label = WIDGET_LABEL(commands, VALUE='Fraktale Welt', FONT=headfont, /ALIGN_LEFT)
space = WIDGET_LABEL(commands, VALUE=' ', FONT=tinyfont)
iter_dummy = WIDGET_LABEL(commands, VALUE='fractional', FONT=boldfont)
iter_dummy1 = WIDGET_LABEL(commands, VALUE='Brownian motion', FONT=boldfont)

help_button = WIDGET_BUTTON(commands, VALUE='Erklrung', FONT="variable")

dummy = WIDGET_LABEL(commands, VALUE="_______________________", FONT=boldfont)
space1 = WIDGET_LABEL(commands, VALUE=' ', FONT=tinyfont)
parm_label = WIDGET_LABEL(commands, VALUE='Fraktalparameter', FONT=boldfont, /ALIGN_LEFT)
beta = WIDGET_BASE(commands, ROW=1)
beta_dummy = WIDGET_LABEL(beta, VALUE="Spektralindex.:     ", FONT=normalfont)
beta_form = WIDGET_TEXT(beta, VALUE='3.0', /EDITABLE, XSIZE=6, FONT=normalfont)
seed = WIDGET_BASE(commands, ROW=1)
seed_dummy = WIDGET_LABEL(seed, VALUE="Zufallsstartwert:   ", FONT=normalfont)
seed_form = WIDGET_TEXT(seed, VALUE='23', /EDITABLE, XSIZE=6, FONT=normalfont)

dummy1 = WIDGET_LABEL(commands, VALUE="_______________________", FONT=boldfont)
space2 = WIDGET_LABEL(commands, VALUE=' ', FONT=tinyfont)
parm1_label = WIDGET_LABEL(commands, VALUE='Darstellung', FONT=boldfont, /ALIGN_LEFT)
select = ['Moleklwolke', 'atmosphrische Wolke', 'fraktales Gebirge', 'fraktale Melodie', 'Sprache/Zwitschern', 'akustische Wellenform  ']
oplist = WIDGET_DROPLIST(commands, VALUE=select, /ALIGN_LEFT, FONT=buttonfont)

scal = WIDGET_BASE(commands, ROW=1)
scal_dummy = WIDGET_LABEL(scal, VALUE='Skala:                  ', FONT=normalfont)
scal_form = WIDGET_TEXT(scal, VALUE='1.0', /EDITABLE, XSIZE=6, FONT=normalfont)
dparm = WIDGET_BASE(commands, ROW=1)
dparm_dummy = WIDGET_LABEL(dparm, VALUE='Hhe:                  ', FONT=normalfont)
dparm_form = WIDGET_TEXT(dparm, VALUE='1.0', /EDITABLE, XSIZE=6, FONT=normalfont)

plot_button = WIDGET_BUTTON(commands, VALUE='Start', FONT=startfont)

dummy2 = WIDGET_LABEL(commands, VALUE="_______________________", FONT=boldfont)
status = WIDGET_LABEL(commands, VALUE=" ", /DYNAMIC_RESIZE, FONT=normalfont) 
bye_button = WIDGET_BUTTON(commands, VALUE='Programmende', FONT=buttonfont)

WIDGET_CONTROL, tlb, /REALIZE
set_plot, "X"
device, decomposed=0
device, bypass_translation=0

WIDGET_CONTROL, draw, GET_VALUE=window_index
info = {draw_wid:window_index, status:status, select:oplist, $
	beta:beta_form, seed:seed_form, scal:scal_form, $
	height:dparm_form }
WIDGET_CONTROL, tlb, SET_UVALUE=info
WIDGET_CONTROL, oplist, SET_DROPLIST_SELECT=2
opening, window_index

loadct, 39
; XMANAGER, 'Fractal', tlb, EVENT='Fractal_Event', NO_BLOCK=0
XMANAGER, 'Fractal', tlb, EVENT='Fractal_Event', NO_BLOCK=1
END
