pro fractalfunc, name,beta,seed=s,size=n
; NAME:
;        FRACTALFUNC
; Purpose:
;        This Routine generates a 1dim fBm function
; CALLING SEQUENCE:
;        FRACTALFUNC, name, beta 
; INPUT:
;        name:  name of the image
;        beta: exponent in spectral density function
; OPTIONAL KEYWORD INPUT: 
;        seed: Seed value for RANDOMU function
;        size: size of the image in pixels (must be even!)
;
;  Written by V. Ossenkopf (8/2000)
; 
;  original 2-D version by M. Zielinski
;

on_error, 2     ; return to caller
if (n_params() lt 2) then begin
   print,'SYNTAX:  FRACTALFUNC, name, beta [,seed=s, size=n] '
   retall
endif
if n_elements(s) lt 1 then print, "seed not defined, chosen arbitrary"
if (n_elements(n) eq 1) then begin
 if ( (long(n) mod 2) eq 1) then begin
  print,"WARNING: size must be even! Set to ",long(n)+1
  ndim=long(n)+1
 endif else ndim=long(n) 
endif else begin
 print,"size not defined, set to 128"
 ndim=128L
endelse

data=dcomplexarr(ndim)
fndim=double(ndim)

if beta eq 1.0 then ampli=1.0 $
else ampli=(beta-1.)/fndim/(fndim^(beta-1.)-2.^(beta-1.))

; *********  positive Werte *************************
for ix=0L,ndim/2 do begin

  if (ix eq 0) then phi=0. else phi=2*!pi*randomu(s)
 
  frequenz=float(ix)
  if (frequenz eq 0.) then begin
    amp=0.
  endif else begin
   amp=sqrt(double(ampli/(frequenz/fndim)^beta))
  endelse

  data(ix)=dcomplex(amp*cos(phi),amp*sin(phi))

endfor


; ********* negative Werte ***********************
for ix=1L,ndim/2 do data(ndim-ix) = conj(data(ix))

; ********   inverse FFT  *******

ffdata= fft(data,1)

; ****  rearange array order ****

ffdata=shift(ffdata,ndim/2)

name=float(ffdata)
return
end
 
