pro fractalim, name,beta,seed=s,size=n
; NAME:
;        FRACTALIM
; Purpose:
;        This Routine generates a 2dim image of a fractal cloud 
;        (or landscape, if plotted using 'surface') with fractal index beta
; CALLING SEQUENCE:
;        FRACTALIM, name, beta 
; INPUT:
;        name:  name of the image
;        beta: exponent in spectral density function
; OPTIONAL KEYWORD INPUT: 
;        seed: Seed value for RANDOMU function
;        size: size of the image in pixels (must be even!)
;        
;  Written by M. Zielinsky (10/96)
; 
;  original FORTRAN-version by J. Stutzki
;
;  modified (9/97): image not squared

on_error, 2     ; return to caller
if (n_params() lt 2) then begin
   print,'SYNTAX:  FRACTALIM, name, beta (,seed=s, size=n) '
   retall
endif
if n_elements(s) lt 1 then print, "seed not defined, chosen arbitrary"
if (n_elements(n) eq 1) then begin
 if ( (fix(n) mod 2) eq 1) then begin
  print,"WARNING: size must be even! Set to ",fix(n)+1
  ndim=fix(n)+1
 endif else ndim=fix(n) 
endif else begin
 print,"size not defined, set to 128"
 ndim=128
endelse

data=dcomplexarr(ndim,ndim)
fndim=double(ndim)

if beta eq 1.0 then ampli=1.0 $
else ampli=(beta-1.)/2.^beta/(beta*fndim^(beta-1.)-1.)

; *********  1. quadrant *************************
for ix=0,ndim/2 do begin
 for iy=0,ndim/2 do begin

  if((ix eq 0) or (ix eq ndim/2)) $
  and ((iy eq 0) or (iy eq ndim/2)) then begin
   phi=0.
  endif else begin
   phi=2*!pi*randomu(s)
  endelse   
 
  fix=float(ix)
  fiy=float(iy)

  frequenz=sqrt(fix*fix+fiy*fiy) 
  if (frequenz eq 0.) then begin
    amp=0.
  endif else begin
   amp=sqrt(double(ampli/(frequenz/fndim)^beta))
  endelse

  data(ix,iy)=dcomplex(amp*cos(phi),amp*sin(phi))
 endfor
endfor

; **********  2.quadrant  ***********************

for ix=1,ndim/2 do begin
 for iy=1,ndim/2 do begin
   data(ndim-ix,ndim-iy) = conj(data(ix,iy))
 endfor
endfor

; ********* 3. und 4. quadrant ***********************
for ix=1,ndim/2 do begin
   data(ndim-ix,0) = conj(data(ix,0))
   data(0,ndim-ix) = conj(data(0,ix))
endfor

for ix=1,ndim/2-1 do begin
 for iy=1,ndim/2-1 do begin

  phi=2.*!pi*randomu(s)

  fix=float(ix)
  fiy=float(iy)

  frequenz=sqrt(fix*fix+fiy*fiy)
  if (frequenz eq 0.) then begin
    amp=0.
  endif else begin
   amp=sqrt(double(ampli/(frequenz/fndim)^beta))
  endelse

  data(ndim-ix,iy)=dcomplex(amp*cos(phi),amp*sin(phi))
  data(ix,ndim-iy)=conj(data(ndim-ix,iy))
 endfor
endfor


; ********   inverse FFT  *******

ffdata= fft(data,1)

; ****  rearange array order ****


ffdata=shift(ffdata,0,ndim/2)
ffdata=shift(ffdata,ndim/2,0)

name=float(ffdata)
return
end
 
