; Fraktale-Welt: V. Ossenkopf 2000
; Routines for MIDI file output

; Return variable length encoded integer as used in MIDI files
function  varlengthint, i,length

tmparr=bytarr(4)
mask1='7F'XB
mask2='80'XB
lmask='1FFFFF'XL
j=long(i)

for it=3,0,-1 do begin
  tmparr[it]=byte(j) AND mask1
  j=ishft(j,-7) AND lmask
endfor

length=1L
for it=3,0,-1 do if tmparr[it] ne 0 then length=long(4-it)
retarr=bytarr(length)

for it=0,length-1 do retarr[it]=tmparr[it+4-length] OR mask2
retarr[length-1]=tmparr[3]

return, retarr
end

; Create the default header for the MIDI file
function stdheader, length

mthd=byte('MThd')
hdlength=byte(6L,0,4)
byteorder, hdlength, /htonl
hdform=byte(0,0,2)
byteorder, hdform, /htons
hdtrck=byte(1,0,2)
byteorder, hdtrck, /htons
hdbeat=byte(96,0,2)
byteorder, hdbeat, /htons

mtrk=byte('MTrk')
name=[[0B,'FF'XB,'01'XB],varlengthint(21,l1),byte('Random MIDI Generator')]
l1=n_elements(name)
auth=[[0B,'FF'XB,'02'XB],varlengthint(16,l2),byte('Volker Ossenkopf')]
l2=n_elements(auth)
time=[0B,'FF'XB,'58'XB,'04'XB,'04'XB,'02'XB,'18'XB,'08'XB]
l3=n_elements(time)
tempo=[0B,'FF'XB,'51'XB,'03'XB,'07'XB,'A1'XB,'20'XB]
l4=n_elements(tempo)
instr=[0B,'C0'XB,01B]
l5=n_elements(instr)
lend=4

tklength=byte(long(length+l1+l2+l3+l4+l5+lend),0,4)
byteorder, tklength, /htonl

array=[mthd,hdlength,hdform,hdtrck,hdbeat,mtrk,tklength,name,auth,$
       time,tempo,instr]

return, array
end

; Create standard closing of the MIDI file
function stdend

array=[0B,'FF'XB,'2F'XB,'00'XB]
return, array
end

; Transform the data field into MIDI encoding
function datafield, field, duration, scal=scal

if not keyword_set(scal) then scal=1.0
if scal gt 2.0 then scal=2.0

; Normalize
mmin=min(field)
mmax=max(field)

bfield=byte((field-mmin)*36.0*scal/(mmax-mmin))
shift=7B-(bfield[0] mod 7B)
bfield=bfield+shift
plotmidi, bfield
translate=[0B,2B,4B,5B,7B,9B,11B]
bfield=12B*(bfield/7B)+translate[bfield mod 7B] +12B*byte(5.0-2.0*scal)

strength=64B
onbyte='90'XB
offbyte='80'XB
mask='7F'XB
deltat=varlengthint(duration,dummy)
long=varlengthint(byte(1.5*duration),dummy)

tone=bfield[0] AND mask
array=[0B,onbyte,tone,strength]
for i =1,n_elements(field)-1 do begin
  if (i mod 4) eq 1 then array=[array,long,offbyte,tone,strength] $
  else array=[array,deltat,offbyte,tone,strength]
  tone=bfield[i] AND mask
  array=[array,0B,onbyte,tone,strength]
endfor
long=varlengthint(2B*duration,dummy)
array=[array,long,offbyte,tone,strength]

return, array
end

; Plot the notes corresponding to the MIDI file
pro plotmidi, bfield

usersym, [-1,-0.7,0,0.7,0.7,0.7,1,1,0.7,0,-0.7,-1],$
         [0,0.7,1,0.7,0,4,4,0,-0.7,-1,-0.7,0], /fill
nn=n_elements(bfield)
n0=-0.2*nn
plot, bfield, psym=8, xrange=[n0,nn],xstyle=5, ystyle=4, /noclip, symsize=50.0/max(bfield)

; Notenlinien
sc=[n0,nn-1]
cent=byte(total(bfield)/nn)
line=(cent/7B)*7B-7B
for i=1,5 do begin
 line=line+2B
 oplot, sc, [line,line]
endfor
!p.charsize=292.0/max(bfield)
xyouts, n0,line-8.0 , "!20.!3"
!p.charsize=1.0

return
end

; General MIDI caller routine
pro midifile, name, field, duration, scal=scal

feld=datafield(field,duration,scal=scal)
length=n_elements(feld)
array=[stdheader(length),feld,stdend()]

get_lun, lun
openw, lun, name, /nostdio
writeu, lun, array
free_lun, lun

return
end

